import { printSchemaWithDirectives } from '@graphql-toolkit/common';
import { parse } from 'graphql';
import { transformASTSchema, Parser } from 'relay-compiler';
import { transform as transform$1 } from 'relay-compiler/lib/transforms/SkipRedundantNodesTransform';
import { transform as transform$2 } from 'relay-compiler/lib/transforms/InlineFragmentsTransform';
import { transform } from 'relay-compiler/lib/transforms/ApplyFragmentArgumentTransform';
import { transformWithOptions } from 'relay-compiler/lib/transforms/FlattenTransform';

const RelayCreate = require('relay-compiler/lib/core/Schema');
const GraphQLCompilerContext = require('relay-compiler/lib/core/GraphQLCompilerContext');
const { print } = require('relay-compiler/lib/core/GraphQLIRPrinter');
const plugin = (schema, documents, 
// eslint-disable-next-line @typescript-eslint/no-unused-vars
config) => {
    // @TODO way for users to define directives they use, otherwise relay will throw an unknown directive error
    // Maybe we can scan the queries and add them dynamically without users having to do some extra stuff
    // transformASTSchema creates a new schema instance instead of mutating the old one
    const adjustedSchema = RelayCreate.create(printSchemaWithDirectives(transformASTSchema(schema, [
        /* GraphQL */ `
          directive @connection(key: String!) on FIELD
          directive @client on FIELD
        `,
    ])));
    const documentAsts = documents.reduce((prev, v) => {
        return [...prev, ...v.content.definitions];
    }, []);
    const relayDocuments = Parser.transform(adjustedSchema, documentAsts);
    const fragmentCompilerContext = new GraphQLCompilerContext(adjustedSchema).addAll(relayDocuments);
    const fragmentDocuments = fragmentCompilerContext
        .applyTransforms([transform, transformWithOptions({ flattenAbstractTypes: false }), transform$1])
        .documents()
        .filter(doc => doc.kind === 'Fragment');
    const queryCompilerContext = new GraphQLCompilerContext(adjustedSchema)
        .addAll(relayDocuments)
        .applyTransforms([transform, transform$2, transformWithOptions({ flattenAbstractTypes: false }), transform$1]);
    const newQueryDocuments = queryCompilerContext.documents().map(doc => ({
        filePath: 'optimized by relay',
        content: parse(print(adjustedSchema, doc)),
    }));
    const newDocuments = [
        ...fragmentDocuments.map(doc => ({
            filePath: 'optimized by relay',
            content: parse(print(adjustedSchema, doc)),
        })),
        ...newQueryDocuments,
    ];
    documents.splice(0, documents.length);
    documents.push(...newDocuments);
    return {
        content: '',
    };
};

export { plugin };
